/**
 * This file is part of my seminar project.
 * Following code, along with other files in this project,
 * realizes the visualization of A* algorithm.
 * 
 * @author Teon Banek
 */

package hr.fer.seminar;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * This class listens to mouse events passed to {@link TileMap}. It is used to
 * make changes to tiles according to user input. Allowed changes are: setting
 * tiles to (un)block, setting start tile and goal tile.
 * 
 * @author Teon Banek
 * 
 */
public class TileMapMouseListener extends MouseAdapter {

	private TileMap tileMap;

	/**
	 * Default constructor.
	 * 
	 * @param tileMap
	 *            - {@link TileMap} being listened.
	 */
	public TileMapMouseListener(TileMap tileMap) {
		this.tileMap = tileMap;
	}

	@Override
	public void mousePressed(MouseEvent e) {
		int mapX = e.getX() / Tile.WIDTH;
		int mapY = e.getY() / Tile.HEIGHT;
		performAction(mapX, mapY, e.getModifiersEx());
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		if (e.getX() >= tileMap.getMapWidth() * Tile.WIDTH || e.getX() < 0
				|| e.getY() >= tileMap.getMapHeight() * Tile.HEIGHT
				|| e.getY() < 0) {
			return;
		}
		int mapX = e.getX() / Tile.WIDTH;
		int mapY = e.getY() / Tile.HEIGHT;
		performAction(mapX, mapY, e.getModifiersEx());
	}

	/**
	 * Perfoms the action on given {@link TileMap} coordinates based on button
	 * pressed.
	 * 
	 * @param mapX
	 *            - x coordinate of a {@link Tile} in {@link TileMap}.
	 * @param mapY
	 *            - y coordinate of a {@link Tile} in {@link TileMap}.
	 * @param button
	 *            - {@link MouseEvent} extended mask.
	 */
	private void performAction(int mapX, int mapY, int button) {
		if (button == MouseEvent.BUTTON1_DOWN_MASK) {
			if (tileMap.isPickStart()) {
				tileMap.setStartTile(mapX, mapY);
			} else if (tileMap.isPickGoal()) {
				tileMap.setGoalTile(mapX, mapY);
			} else {
				blockTile(mapX, mapY, true);
			}
		} else if (button == MouseEvent.BUTTON3_DOWN_MASK) {
			blockTile(mapX, mapY, false);
		}
	}

	/**
	 * Sets the tile at given coordinates to given blocking flag.
	 * 
	 * @param x
	 *            - x coordinate of a {@link Tile} on {@link TileMap}.
	 * @param y
	 *            - y coordinate of a {@link Tile} on {@link TileMap}.
	 * @param blocking
	 *            - flag to be set.
	 */
	private void blockTile(int x, int y, boolean blocking) {
		Tile tile = tileMap.getTile(x, y);
		if (tile == null || tile.isBlocking() == blocking) {
			return;
		}
		tile.setBlocking(blocking);
		tileMap.updateUI();
	}
}